#include <windows.h>
#include <string.h>
#include "cipher.h"

#define FILE_NOT_EXIST            -1 
#define FILE_OPEN_CREATE_FAIL     -2
#define DEVICE_CONNECT_FAIL       -3
#define TIME_OUT                  -4

HANDLE hDLLInst = 0;

static BOOL bConnect (int nPort, int nBaud);
static int nReadData (void);
static void OnLineMsg (int nAction, int nCount);

static char *szComPort[] = {"COM1", "COM2", "COM3", "COM4"};
static HANDLE hFile;
static HANDLE hComm;               // the handle of the opened Comm Device.
static char   cInBuf [256];        // Input buffer for receiving data.
static char   szBuf [256];         // Record Buffer.

//-------------------------------------------------------------------------//
BOOL WINAPI DllMain (HANDLE hModule, DWORD dwFunc, LPVOID lpNot)
    {
    hDLLInst = hModule;

    switch (dwFunc)
        {
        case DLL_PROCESS_ATTACH:
        case DLL_PROCESS_DETACH:
        default:
             break;
        }
    return TRUE;
    }

//-------------------------------------------------------------------------//
DllExport int WINAPI UPLOAD (LPSTR lpFileName, int nPort, int nBaud)
   {   
   char      cFileMode;
   int       nRecord=0;
   LPSTR     lpFile = lpFileName;
         
   while (*lpFile++)
      {
      if (*lpFile == ',')
         {
         cFileMode = *(lpFile+1);
         *lpFile = 0;
         break;
         }
      }
      
   if (cFileMode == 'A' || cFileMode == 'a')
      { 
      hFile = CreateFile (lpFileName, GENERIC_READ | GENERIC_WRITE, 0, NULL,
                          OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
      if (hFile == INVALID_HANDLE_VALUE)
         { 
         if (MessageBox (NULL, "File does not exist, create it?", 
                     "Error", MB_YESNO) == IDNO)
             return FILE_NOT_EXIST;
         hFile = CreateFile (lpFileName, GENERIC_READ | GENERIC_WRITE, 0,
                             NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);
         }                                                     
      else                                   // move to the end of the file.
         {
         SetFilePointer (hFile, 0, NULL, FILE_END);
         }
      }
   else
      {  
      hFile = CreateFile (lpFileName, GENERIC_READ | GENERIC_WRITE, 0,
                          NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
      }                
       
   if (hFile == INVALID_HANDLE_VALUE)
      { 
      MessageBox (NULL, "File open/create failed!", "Error", MB_OK);
      return FILE_OPEN_CREATE_FAIL;
      }
                             
   // Open Comm Port & Read Data.
   if ( bConnect (nPort-1, nBaud) )          // if connect OK.
      {
      nRecord = nReadData ();                // start to read data.
      CloseHandle (hComm);                   // close the COM Port.
      CloseHandle (hFile);                   // close the opened file.
      }
   else
      {
      CloseHandle (hFile);                   // close the opened file.
      return DEVICE_CONNECT_FAIL;
      }
       
   if (nRecord > 0)
      {
      char  szMsg[64];
      wsprintf (szMsg, "Done : %d records received.", nRecord);
      MessageBox (NULL, szMsg, "Upload Data", MB_OK);
      }
   else
      MessageBox (NULL, "No data received!", "Upload Data", MB_OK);

   return nRecord;
   }                                                                         
   
//-------------------------------------------------------------------------//
/*
static BOOL bConnect (void)
    {
    static DCB         dcb;
    static COMMCONFIG  cc;
    HWND   hWnd;

    if ((hComm = CreateFile ("COM1", GENERIC_READ | GENERIC_WRITE, 0, NULL,
                            OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE)
//        OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL)) == INVALID_HANDLE_VALUE)
        {
        MessageBox (NULL, "Error opening COM port", "Error", MB_OK);
        return FALSE;
        }

    SetupComm (hComm, 256, 256);       // allocate transmit & receive buffer.
    BuildCommDCB ("38400,N,8,1", &dcb);         // fill device control block.

    cc.dwSize = sizeof (COMMCONFIG);
    cc.wVersion = 0x100;                        // WIN95_COMMCONFIG_VERSION.
    hWnd = GetActiveWindow();
    if (CommConfigDialog ("COM1", hWnd, &cc))
        {
        dcb.BaudRate = cc.dcb.BaudRate;
        dcb.Parity   = cc.dcb.Parity;
        dcb.ByteSize = cc.dcb.ByteSize;
        dcb.StopBits = cc.dcb.StopBits;
        }
    UpdateWindow (hWnd);

    if (SetCommState (hComm, &dcb) < 0)
        {
        MessageBox (NULL, "SetCommState Failed", "Error", MB_OK);
        return FALSE;
        }

    return TRUE;
    }
*/
//-------------------------------------------------------------------------//
static BOOL bConnect (int nPort, int nBaud)
    {
    static DCB   dcb;

    if ((hComm = CreateFile (szComPort [nPort], GENERIC_READ | GENERIC_WRITE,
                  0, NULL, OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE)
        {
        MessageBox (NULL, "Fail to open COM port", "Error", MB_OK);
        return FALSE;
        }

    SetupComm (hComm, 256, 256);       // allocate transmit & receive buffer.
    BuildCommDCB ("9600,N,8,1", &dcb);          // fill device control block.

    switch (nBaud)
        {
        case 38400: dcb.BaudRate = CBR_38400;   break;
        case 19200: dcb.BaudRate = CBR_19200;   break;
        case 9600:  dcb.BaudRate = CBR_9600;    break;
        case 4800:  dcb.BaudRate = CBR_4800;    break;
        case 2400:  dcb.BaudRate = CBR_2400;    break;
        case 1200:  dcb.BaudRate = CBR_1200;    break;
        default:    dcb.BaudRate = CBR_9600;    break;
        }

    if (SetCommState (hComm, &dcb) < 0)
        {
        CloseHandle (hComm);                    // close the COM port.
        return FALSE;
        }

    return TRUE;
    }

//-------------------------------------------------------------------------//
static int nReadData (VOID)
   {
   DWORD   i, nChar, nCount;
   char    cRet;
   int     nRecord=0;
   DWORD   dwTime0;
   static  COMMTIMEOUTS TimeOuts;
                        
   TimeOuts.ReadIntervalTimeout = MAXDWORD;
   TimeOuts.ReadTotalTimeoutMultiplier = 0;
   TimeOuts.ReadTotalTimeoutConstant = 0;
   TimeOuts.WriteTotalTimeoutMultiplier = 5;
   TimeOuts.WriteTotalTimeoutConstant = 50;
   SetCommTimeouts (hComm, &TimeOuts);          // check dwProvCapabilities.

   OnLineMsg (0, 0);
   while (TRUE)
      {                               
      // READ
R1:   WriteFile (hComm, "READ\r", 5, &nChar, NULL);
      dwTime0 = GetTickCount ();
      nCount = 0;                          // reset current char counter.
      cRet = 0;
      
      while (cRet != '\r')                 // while not get the return char.
         {                                 // keep on reading.
         if (ReadFile (hComm, cInBuf, 255, &nChar, NULL))
            {
            for (i=0; i<nChar; i++)
                szBuf [nCount++] = cInBuf [i];
            cRet = cInBuf[i-1];
            }
         if (GetTickCount() - dwTime0 > 3000)     // check if time out.
            {                            
            OnLineMsg (2, 0);
            MessageBox (NULL, "Time out!", "Error", MB_OK);
            return TIME_OUT;
            }
         }
      szBuf [nCount] = 0;

      if (lstrcmp (szBuf, "OVER\r") == 0)  // no more data.
          break;
          
      if (lstrcmp (szBuf, "NAK\r") == 0)   // if command isn't correct.
          goto R1;
      else                                 // data is correct.
          {
          szBuf [nCount++] = '\n';         // add newline char.
          szBuf [nCount] = 0;
          WriteFile (hFile, szBuf, nCount, &nChar, NULL);   // write to file.
          nRecord++;                       // increase record number.
          OnLineMsg (1, nRecord);
          }
      
      for (i=0; i<=nCount; i++)            // clear buffer.
          szBuf [i] = 0;                   
             
      // REMOVE
REDO: WriteFile (hComm, "REMOVE\r", 7, &nChar, NULL);
      dwTime0 = GetTickCount ();
      nCount = 0;                          // reset current character counter.
      cRet = 0;
            
      while (cRet != '\r')                 // while not get the return char.
         {                                 // keep on reading.
         if (ReadFile (hComm, cInBuf, 255, &nChar, NULL))
            {
            for (i=0; i<nChar; i++)
                szBuf [nCount++] = cInBuf [i];
            cRet = cInBuf[i-1];
            } 

         if (GetTickCount() - dwTime0 > 3000)     // check if time out.
            {
            OnLineMsg (2, 0);
            MessageBox (NULL, "Time out!", "Error", MB_OK);
            if (nRecord > 0)
                return nRecord;
            return TIME_OUT;
            }             
         }
      szBuf [nCount] = 0;
       
      if (lstrcmp (szBuf, "OVER\r") == 0)  // no more data.
          break;
          
      if (lstrcmp (szBuf, "NAK\r") == 0)   // if command error.
          goto REDO;    
                           
      for (i=0; i<nCount; i++)             // clear buffer.
          szBuf [i] = 0;                   
      }
 
   OnLineMsg (2, 0);
   return nRecord;
   }

//-------------------------------------------------------------------------//
static void OnLineMsg (int nAction, int nCount)
   {
   static HBITMAP  hBitmap;
   static HDC      hMemDC;
   static char     szMsg[24]="Data received : 0   ";
   static int      nW, hx, hy;
   HDC             hdc;

   hdc = GetDC (NULL);

   if (nAction == 0)                            // backup the original image.
       {
       SIZE  size;
       hx = GetDeviceCaps (hdc, HORZRES) / 2;
       hy = GetDeviceCaps (hdc, VERTRES) / 2;
       GetTextExtentPoint32 (hdc, szMsg, lstrlen(szMsg), &size);
       nW = size.cx + 20;
       hMemDC = CreateCompatibleDC (hdc);
       hBitmap = CreateCompatibleBitmap (hdc, nW, 40);
       SelectObject (hMemDC, hBitmap);
       BitBlt (hMemDC, 0, 0, nW, 40, hdc, hx-nW/2, hy-20, SRCCOPY);
       }
   else if (nAction == 1)                       // show record number.
       {
       HBRUSH    hBrush, hOldBrush;
       RECT      rc;

       hBrush = CreateSolidBrush (RGB (255, 255, 100));
       SetBkMode (hdc, TRANSPARENT);
       hOldBrush = SelectObject (hdc, hBrush);
       SetRect (&rc, hx-nW/2, hy-20, hx+nW/2, hy+20);
       wsprintf (szMsg, "Data received : %d", nCount);
       Rectangle (hdc, rc.left, rc.top, rc.right, rc.bottom);
       DrawText (hdc, szMsg, -1, &rc, DT_SINGLELINE | DT_VCENTER | DT_CENTER);
       SelectObject (hdc, hOldBrush);
       DeleteObject (hBrush);
       }
   else if (hBitmap)                            // restore.
       {
       BitBlt (hdc, hx-nW/2, hy-20, nW, 40, hMemDC, 0, 0, SRCCOPY);
       DeleteDC (hMemDC);
       DeleteObject (hBitmap);
       hBitmap = NULL;
       }

   ReleaseDC (NULL, hdc);
   }
//-------------------------------------------------------------------------//

